class Geom::PolygonMesh

def add_lathe(pts, center, dir, angle, numsegments) # Copyright 2004-2005, @Last Software, Inc. (I changed it a little)

numpts = pts.length
     if( numpts < 2 )
         raise ArgumentError, "At least two points required", caller
     end

     da    = angle / numsegments
     tform = Geom::Transformation.rotation center, dir, da

     index_array = []
     for i in 0...numsegments+1 do
         indices = []
         for pt in pts do
             indices.push( self.add_point(pt) )
             pt.transform!(tform)
         end
         index_array.push indices
     end

     i1 = index_array[0]
     for i in 0...numsegments+1 do
         i2 = index_array[i]

         for j in 0...numpts do
             k = (j+1) % numpts
             self.add_polygon -i1[j], i2[j], i2[k], -i1[k]
         end

         i1 = i2
     end

     end

end

class PipeTool

@grad = '6"'
@gseg = "24"

@shei = "2'"

@ccrad = "2'"
@ccseg = "24"
@cang  = "90"

@rerad = '3"'
@rhei  = "1'"

def PipeTool.makecircle(center, normal, radius, numseg) # Copyright 2004, @Last Software, Inc.
    axes   = Geom::Vector3d.new(normal).axes
    center = Geom::Point3d.new(center)
    xaxis  = axes[0]
    yaxis  = axes[1]

    xaxis.length = radius
    yaxis.length = radius

    da  = (Math::PI * 2) / numseg
    pts = []
    for i in 0...numseg do
        angle = i * da
        cosa  = Math.cos(angle)
        sina  = Math.sin(angle)
        vec   = Geom::Vector3d.linear_combination(cosa, xaxis, sina, yaxis)
        pts.push(center + vec)
    end

    pts.push(pts[0].clone)

    pts
end

def PipeTool.straight_pipe

model    = Sketchup.active_model
entities = model.entities
prompts  = ["Radius", "Height", "Segments"]
defaults = [@grad, @shei, @gseg]
input    = UI.inputbox(prompts, defaults, "Make A Straight Pipe")

if input

     @grad, @shei, @gseg = input

     rad = input[0].to_l
     hei = input[1].to_l
     seg = input[2].to_i

     model.start_operation("Make A Straight Pipe")

     puts "Straight pipe created:"
     puts "| The radius of the pipe is:     "+rad.to_s
     puts "| The height of the pipe is:     "+hei.to_s
     puts "| There were "+seg.to_s+" segments in the pipe"

     pipe                        = model.definitions.add "Straight Pipe "
     pipe.behavior.is2d          = true
     pipe.behavior.cuts_opening  = true
     e                           = pipe.entities
     centerpoint                 = Geom::Point3d.new
     center                      = Geom::Vector3d.new 0,0,1
     normal                      = center.normalize!
     circle                      = e.add_circle centerpoint, normal, rad, seg
     face                        = e.add_face circle
     height                      = face.pushpull -hei
     pos1                        = Geom::Point3d.new(0,0,0)
     pos2                        = Geom::Point3d.new(0,0,hei)
     point1                      = e.add_cpoint pos1
     point2                      = e.add_cpoint pos2
     place                       = model.place_component pipe, true

     model.commit_operation

end

end # PipeTool.straight_pipe

def PipeTool.curved_pipe

model    = Sketchup.active_model
entities = model.entities

prompts  = ["Pipe Radius", "Curve Radius", "Pipe Segments", "Curve Segments", "Angle (Degrees)"]
defaults = [@grad, @ccrad, @gseg, @ccseg, @cang]
input    = UI.inputbox(prompts, defaults, "Make A Curved Pipe")

if input

     @grad, @ccrad, @gseg, @ccseg, @cang = input

     prad = input[0].to_l
     crad = input[1].to_l
     pseg = input[2].to_i
     cseg = input[3].to_i
     ang  = input[4].to_f

     model.start_operation("Make A Curved Pipe")

     puts "Curved pipe created:"
     puts "| The radius of the pipe is:      "+prad.to_s
     puts "| The radius of the curve is:     "+crad.to_s
     puts "| There were "+pseg.to_s+" segments in the pipe"
     puts "| There were "+cseg.to_s+" segments in the curve"
     puts "| The angle of the curve is:      "+ang.to_s+""

     pipe                        = model.definitions.add "Curved Pipe "
     pipe.behavior.is2d          = true
     pipe.behavior.cuts_opening  = true
     e                           = pipe.entities
     centerpoint                 = Geom::Point3d.new 0,0,0
     point3                      = e.add_cpoint centerpoint
     vector                      = Geom::Vector3d.new 0,0,1
     normal                      = vector.normalize!
     cr                          = Geom::Point3d.new crad,0,0
     point1                      = e.add_cpoint cr
     vector                      = Geom::Vector3d.new(0,1,0)
     circle                      = PipeTool.makecircle(centerpoint, normal, prad, pseg)
     mesh                        = Geom::PolygonMesh.new
     lathe                       = mesh.add_lathe(circle, cr, vector, ang.degrees, cseg)
     faces                       = e.add_faces_from_mesh( mesh, 12 )
     group                       = e.add_group
     ge                          = group.entities
     circle                      = ge.add_circle centerpoint, normal, prad, pseg
     face                        = ge.add_face circle
     point                       = ge.add_cpoint centerpoint
     tr1                         = group.transformation
     tr2                         = Geom::Transformation.rotation cr, vector, ang.degrees
     transformation              = group.transform!(tr1.inverse)
     transformation              = group.transform!(tr2)
     transformation              = group.transform!(tr1)
     transformation              = group.explode
     circle                      = e.add_circle centerpoint, normal, prad, pseg
     face                        = e.add_face circle

     e.each{|e|
         if e.is_a? Sketchup::Face
             e.reverse!
         end

         if e.is_a? Sketchup::Edge
             if e.length.to_f == 0
                 e.erase!
             end
         end
     }
	 
     place = model.place_component pipe, true

     model.commit_operation

end # if
end # PipeTool.curved_pipe

def PipeTool.reducer_pipe

 model    = Sketchup.active_model
 entities = model.entities

 prompts  = ["Start radius", "End radius", "Height", "Segments"]
 defaults = [@grad, @rerad, @rhei, @gseg]
 input    = UI.inputbox(prompts, defaults, "Make A Reducer Pipe")

     if input

         @rsrad, @grad, @rhei, @gseg = input

         srad = input[0].to_l
         erad = input[1].to_l
         hei  = input[2].to_l
         seg  = input[3].to_i

         model.start_operation("Make A Reducer Pipe")

         puts "A reducer pipe created:"
         puts "| The starting radius of the pipe is:  "+srad.to_s
         puts "| The ending radius of the pipe is:    "+erad.to_s
         puts "| The height of the pipe is:           "+hei.to_s
         puts "| There were "+seg.to_s+" segments in the pipe"

         pipe                       = model.definitions.add "Reducer Pipe "
         pipe.behavior.is2d         = true
         pipe.behavior.cuts_opening = true
         entities                   = pipe.entities
         centerpoint                = Geom::Point3d.new
         center                     = Geom::Vector3d.new 0,0,1
         normal                     = center.normalize!
         circle                     = entities.add_circle centerpoint, normal, erad, seg
         face                       = entities.add_face circle
         height                     = face.pushpull -hei
         pos1                       = Geom::Point3d.new(0,0,0)
         pos2                       = Geom::Point3d.new(0,0,hei)
         point1                     = entities.add_cpoint pos1
         point2                     = entities.add_cpoint pos2
         transform                  = Geom::Transformation.scaling srad.to_f/erad.to_f
         scale                      = entities.transform_entities(transform, circle)
         place                      = model.place_component pipe, true

         model.commit_operation

     end
end # def PipeTool.reducer_pipe

def PipeTool.piperoom

model    = Sketchup.active_model
entities = model.entities

prompts  = ["length (Feet)", "Width (Feet)", "Height (Feet)"]
defaults = ["10","10","10"]
input    = UI.inputbox(prompts, defaults, "Make a pipe room")

     if input

         model.start_operation("Make A Pipe Room")

         len = input[0].to_i #x
         wid = input[1].to_i #y
         hei = input[2].to_i #z

         puts "A room to hold all new pipes was created:"
         puts "| The length of the room is:   "+len.to_s+" feet"
         puts "| The width of the room is:    "+wid.to_s+" feet"
         puts "| The height of the room is:   "+hei.to_s+" feet"

         group = entities.add_group
         ent   = group.entities

         for i in 0..len-1 #x
         i = i*12
         for j in 0..wid-1 #y
             j = j*12
             pts    = []
             pts[0] = Geom::Point3d.new(i,j,0)
             pts[1] = Geom::Point3d.new(i+12,j,0)
             pts[2] = Geom::Point3d.new(i+12,j+12,0)
             pts[3] = Geom::Point3d.new(i,j+12,0)
             face   = ent.add_face pts
             point  = ent.add_cpoint Geom::Point3d.new(i+6,j+6,0)

         end
         end

         for k in 0..wid-1 #y
         k = k*12
         for l in 0..hei-1 #z
             l = l*12
             pts    = []
             pts[0] = Geom::Point3d.new(0,k,l)
             pts[1] = Geom::Point3d.new(0,k+12,l)
             pts[2] = Geom::Point3d.new(0,k+12,l+12)
             pts[3] = Geom::Point3d.new(0,k,l+12)
             face   = ent.add_face pts
             point  = ent.add_cpoint Geom::Point3d.new(0,k+6,l+6)

         end
         end

         for m in 0..len-1 #x
         m = m*12
         for n in 0..hei-1 #z
             n = n*12
             pts    = []
             pts[0] = Geom::Point3d.new(m,0,n)
             pts[1] = Geom::Point3d.new(m+12,0,n)
             pts[2] = Geom::Point3d.new(m+12,0,n+12)
             pts[3] = Geom::Point3d.new(m,0,n+12)
             face   = ent.add_face pts
             point  = ent.add_cpoint Geom::Point3d.new(m+6,0,n+6)

         end
         end

         ent.each{|f|
             if f.is_a? Sketchup::Face
                 f.reverse!
             end
         }

         model.commit_operation

    end # if input

end # def PipeTool.piperoom

def PipeTool.glue
         model     = Sketchup.active_model
         selection = model.selection
     if(selection.count == 2 )and((selection[0].is_a? Sketchup::ComponentInstance)or(selection[0].is_a? Sketchup::Group))and((selection[1].is_a? Sketchup::ComponentInstance)or(selection[1].is_a? Sketchup::Group))

     selection[0].outer_shell(selection[1])

     else
         UI.messagebox("You must first select 2 pipes that you want to glue together")
     end

end

end # class

     toolbar                  = UI::Toolbar.new "Pipes"

     spipe                    = UI::Command.new("Straight Pipe") {PipeTool.straight_pipe}
     spipe.small_icon         = "StraightPipe.png"
     spipe.large_icon         = "StraightPipe.png"
     spipe.tooltip            = "Make A Straight Pipe"
     spipe.status_bar_text    = "Make A Straight Pipe"
     spipe.menu_text          = "Make A Straight Pipe"

     cpipe                    = UI::Command.new("Curved Pipe") {PipeTool.curved_pipe}
     cpipe.small_icon         = "CurvePipe.png"
     cpipe.large_icon         = "CurvePipe.png"
     cpipe.tooltip            = "Make A Curved Pipe"
     cpipe.status_bar_text    = "Make A Curved Pipe"
     cpipe.menu_text          = "Make A Curved Pipe"

     rpipe                    = UI::Command.new("Reducer Pipe") {PipeTool.reducer_pipe}
     rpipe.small_icon         = "Reducer.png"
     rpipe.large_icon         = "Reducer.png"
     rpipe.tooltip            = "Make A Reducer Pipe"
     rpipe.status_bar_text    = "Make A Reducer Pipe"
     rpipe.menu_text          = "Make A Reducer Pipe"

     glue                     = UI::Command.new("Glue") {PipeTool.glue}
     glue.small_icon          = "Glue.png"
     glue.large_icon          = "Glue.png"
     glue.tooltip             = "Glue 2 pipes together"
     glue.status_bar_text     = "Glue 2 pipes together"
     glue.menu_text           = "Glue 2 pipes together"

     piperoom                 = UI::Command.new("Pipe Room") {PipeTool.piperoom}
     piperoom.small_icon      = "Pipe Room.png"
     piperoom.large_icon      = "Pipe Room.png"
     piperoom.tooltip         = "Make a room to hold all new pipes"
     piperoom.status_bar_text = "Make a room to hold all new pipes"
     piperoom.menu_text       = "Make a room to hold all new pipes"

     toolbar                  = toolbar.add_item spipe
     toolbar                  = toolbar.add_item cpipe
     toolbar                  = toolbar.add_item rpipe
     toolbar                  = toolbar.add_item glue
     toolbar                  = toolbar.add_item piperoom
     show                     = toolbar.show

